package de.hsel.itech.db;


import de.hsel.itech.db.pojo.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.AbstractMap;
import java.util.Map;

/**
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.5
 */
public class PaymentDB {

    private Database database;

    PaymentDB(Database database) {
        this.database = database;
    }

    @Nullable
    private PaymentType getType(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePaymentType, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            PaymentType type = new PaymentType(id, rs.getString("name"));
            entry.getValue().close();

            return type;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    public Payment get(long id) {
        Map.Entry<User, PaymentType> metadata = getMetadata(id);
        assert metadata != null;

        String typeName = metadata.getValue().getName();
        if(typeName.equals("PayPal")) {
            return getPayPal(id, metadata.getKey());
        }
        if(typeName.equals("CreditCard")) {
            return getCreditCard(id, metadata.getKey());
        }
        if(typeName.equals("DebitCard")) {
            return getDebitCard(id, metadata.getKey());
        }
        if(typeName.equals("Invoice")) {
            return getInvoice(id, metadata.getKey());
        }

        return null;
    }

    @Nullable
    private PayPalPayment getPayPal(long id, @NotNull User user) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePaymentPayPal, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            PayPalPayment payment = new PayPalPayment(id, user, rs.getString("email"), rs.getString("authCode"));
            entry.getValue().close();

            return payment;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    private CreditCardPayment getCreditCard(long id, @NotNull User user) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePaymentCredit, id);
        assert entry != null;


        try {
            ResultSet rs = entry.getKey();
            CreditCardPayment payment = new CreditCardPayment(id, user, rs.getInt("number"), rs.getString("owner"), rs.getDate("expiration"), rs.getInt("checksum"));
            entry.getValue().close();

            return payment;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }


    @Nullable
    private DebitCardPayment getDebitCard(long id, @NotNull User user) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePaymentDebit, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            DebitCardPayment payment = new DebitCardPayment(id, user, rs.getString("iban"), rs.getString("bic"), rs.getString("owner"));
            entry.getValue().close();

            return payment;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Nullable
    private InvoicePayment getInvoice(long id, @NotNull User user) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tablePaymentInvoice, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            InvoicePayment payment = new InvoicePayment(id, user);
            entry.getValue().close();

            return payment;
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return null;
    }

    @Nullable
    private Map.Entry<User, PaymentType> getMetadata(long id) {
        Map.Entry<ResultSet, Connection> entry = database.getResultSetById(database.tableUserPayment, id);
        assert entry != null;

        try {
            ResultSet rs = entry.getKey();
            Map.Entry<User, PaymentType> metadata = new AbstractMap.SimpleEntry<>(database.user().get(rs.getLong("user")), getType(rs.getLong("type")));

            entry.getValue().close();

            return metadata;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }
}